package com.muyu.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.constant.UserConstants;
import com.muyu.common.core.enums.SysDelFlag;
import com.muyu.common.core.enums.SysNormalDisable;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.SpringUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.datascope.annotation.DataScope;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysDept;
import com.muyu.common.system.domain.SysRole;
import com.muyu.common.system.domain.SysUser;
import com.muyu.system.domain.model.CheckDeptNameUniqueModel;
import com.muyu.system.domain.model.SysDeptPageQueryModel;
import com.muyu.system.domain.vo.TreeSelect;
import com.muyu.system.mapper.SysDeptMapper;
import com.muyu.system.mapper.SysRoleMapper;
import com.muyu.system.service.SysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 *
 * @author muyu
 */
@Service
public class SysDeptServiceImpl extends ServiceImpl<SysDeptMapper, SysDept>
        implements SysDeptService {
    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private SysRoleMapper roleMapper;


    @Override
    public List<SysDept> queryList(SysDeptPageQueryModel sysDeptPageQueryModel) {
        LambdaQueryWrapper<SysDept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(sysDeptPageQueryModel.getDeptName()),SysDept::getDeptName, sysDeptPageQueryModel.getDeptName());
        queryWrapper.eq(StringUtils.isNotEmpty(sysDeptPageQueryModel.getStatus()),SysDept::getStatus,sysDeptPageQueryModel.getStatus());
        queryWrapper.eq(SysDept::getStatus, SysDelFlag.EXIST.getCode());
        return this.list(queryWrapper);
    }


    /**
     * 查询部门管理数据
     *
     * @param sysDeptPageQueryModel 部门信息
     *
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList (SysDeptPageQueryModel sysDeptPageQueryModel) {
        LambdaQueryWrapper<SysDept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(
                StringUtils.isNotNull(sysDeptPageQueryModel.getDeptName()),SysDept::getDeptName, sysDeptPageQueryModel.getDeptName()
        );
        queryWrapper.eq(Objects.nonNull(sysDeptPageQueryModel.getStatus()),SysDept::getStatus,sysDeptPageQueryModel.getStatus());
        queryWrapper.eq(sysDeptPageQueryModel.getDeptId() != null,SysDept::getDeptId,sysDeptPageQueryModel.getDeptId());
        return this.list(queryWrapper);
    }

    /**
     * 查询部门树结构信息
     *
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectDeptTreeList () {
        return buildDeptTreeSelect(
                SpringUtils.getAopProxy(this).selectDeptList(
                        SysDeptPageQueryModel.ofNull()
                )
        );
    }

    /**
     * 构建前端所需要树结构
     *
     * @param depts 部门列表
     *
     * @return 树结构列表
     */
    @Override
    public List<SysDept> buildDeptTree (List<SysDept> depts) {
        List<SysDept> returnList = new ArrayList<>();
        List<Long> tempList = depts.stream().map(SysDept::getDeptId).toList();
        for (SysDept dept : depts) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId())) {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty()) {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     *
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect (List<SysDept> depts) {
        List<SysDept> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     *
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId (Long roleId) {
        SysRole role = roleMapper.selectRoleById(roleId);
        return deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     *
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptById (Long deptId) {
        return this.getById(deptId);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     *
     * @return 子部门数
     */
    @Override
    public int selectNormalChildrenDeptById (Long deptId) {
        return deptMapper.selectNormalChildrenDeptById(deptId);
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     *
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId (Long deptId) {
        int result = deptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     *
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser (Long deptId) {
        int result = deptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }

    /**
     * 校验部门名称是否唯一
     *
     * @param checkDeptNameUniqueModel 检查模型
     * @return 结果
     */
    @Override
    public boolean checkDeptNameUnique (CheckDeptNameUniqueModel checkDeptNameUniqueModel) {
        Assert.notNull(checkDeptNameUniqueModel.getDeptName(), "部门名称不可为空");
        LambdaQueryWrapper<SysDept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SysDept::getDeptName, checkDeptNameUniqueModel.getDeptName());
        // 若不传入部门ID则查询内容为所有部门不唯一
        queryWrapper.eq(Objects.nonNull(checkDeptNameUniqueModel.getParentDeptId()), SysDept::getParentId, checkDeptNameUniqueModel.getParentDeptId());
        //部门的状态必须为正常才可进行判断
        queryWrapper.eq(SysDept::getStatus, SysDelFlag.EXIST.getCode());
        // 如果存在传入的部门则校验除了当前部门之外的其他部门
        queryWrapper.ne(Objects.nonNull(checkDeptNameUniqueModel.getDeptId()),SysDept::getDeptId, checkDeptNameUniqueModel.getDeptId());
        return this.count(queryWrapper) == 0 ? UserConstants.NOT_UNIQUE : UserConstants.UNIQUE;
    }

    /**
     * 校验部门是否有数据权限
     *
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope (Long deptId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            List<SysDept> deptList = SpringUtils.getAopProxy(this)
                    .selectDeptList(SysDeptPageQueryModel.ofToDeptId(deptId));
            if (StringUtils.isEmpty(deptList)) {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     * 新增保存部门信息
     *
     * @param dept 部门信息
     *
     * @return 结果
     */
    @Override
    public void insertDept (SysDept dept) {
        SysDept parentSysDept = this.getById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (SysNormalDisable.isDisable(dept.getStatus())) {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(parentSysDept.getAncestors() + "," + dept.getParentId());
        this.save(dept);
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     *
     * @return 结果
     */
    @Override
    public void updateDept (SysDept dept) {
        SysDept newParentDept = this.getById(dept.getParentId());
        SysDept oldDept = this.getById(dept.getDeptId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept)) {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        this.updateById(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                && !StringUtils.equals("0", dept.getAncestors())) {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal (SysDept dept) {
        String ancestors = dept.getAncestors();
        List<Long> deptIdList = Convert.toLongList(ancestors);
        LambdaUpdateWrapper<SysDept> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(SysDept::getStatus, SysNormalDisable.ENABLE.getCode());
        updateWrapper.in(SysDept::getDeptId, deptIdList);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId       被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren (Long deptId, String newAncestors, String oldAncestors) {
        List<SysDept> children = this.selectChildrenDeptById(deptId);
        for (SysDept child : children) {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (!children.isEmpty()) {
            deptMapper.updateDeptChildren(children);
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     *
     * @return 结果
     */
    @Override
    public void deleteDeptById (Long deptId) {
        LambdaUpdateWrapper<SysDept> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(SysDept::getStatus, SysDelFlag.NOT_EXIST.getCode());
        updateWrapper.eq(SysDept::getDeptId, deptId);
        this.update(updateWrapper);
    }



    /**
     * 递归列表
     */
    private void recursionFn (List<SysDept> list, SysDept t) {
        // 得到子节点列表
        List<SysDept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysDept tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDept> getChildList (List<SysDept> list, SysDept parentSysDept) {
        List<SysDept> childSysDeptList = new ArrayList<>();
        for (SysDept childSysDept : list) {
            if (StringUtils.isNotNull(childSysDept.getParentId()) && childSysDept.getParentId().longValue() == parentSysDept.getDeptId().longValue()) {
                childSysDeptList.add(childSysDept);
            }
        }
        return childSysDeptList;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild (List<SysDept> list, SysDept t) {
        return !getChildList(list, t).isEmpty();
    }

    /**
     * 根据ID查询所有子部门
     *
     * @param deptId 部门ID
     *
     * @return 部门列表
     */
    @Override
    public List<SysDept> selectChildrenDeptById (Long deptId){
        LambdaQueryWrapper<SysDept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.apply("FIND_IN_SET('" + deptId + "', type)");
        return this.list(queryWrapper);
    }
}
