package com.muyu.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.constant.UserConstants;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.SpringUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.PageQueryModel;
import com.muyu.common.datascope.annotation.DataScope;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysOperLog;
import com.muyu.common.system.domain.SysRole;
import com.muyu.common.system.domain.SysUser;
import com.muyu.system.domain.SysRoleDept;
import com.muyu.system.domain.SysRoleMenu;
import com.muyu.system.domain.SysUserRole;
import com.muyu.system.domain.model.SysRolePageQueryModel;
import com.muyu.system.mapper.SysRoleDeptMapper;
import com.muyu.system.mapper.SysRoleMapper;
import com.muyu.system.mapper.SysRoleMenuMapper;
import com.muyu.system.mapper.SysUserRoleMapper;
import com.muyu.system.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 角色 业务层处理
 *
 * @author muyu
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {
    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleDeptMapper roleDeptMapper;

    /**
     * 根据条件分页查询角色数据
     *
     * @param params 角色信息
     *
     * @return 角色数据集合信息
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleList (SysRole params) {
        LambdaQueryWrapper<SysRole> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(params.getRoleId() != null && params.getRoleId() != 0, SysRole::getRoleId, params.getRoleId());
        wrapper.like(StringUtils.isNotBlank(params.getRoleName()), SysRole::getRoleName, params.getRoleName());
        wrapper.eq(StringUtils.isNotBlank(params.getStatus()), SysRole::getStatus, params.getStatus());

        wrapper.like(StringUtils.isNotBlank(params.getRoleKey()), SysRole::getRoleKey, params.getRoleKey());
        wrapper.ge(params.getParams().get("beginTime") != null, SysRole::getCreateTime, params.getParams().get("beginTime"));
        wrapper.le(params.getParams().get("endTime") != null, SysRole::getCreateTime, params.getParams().get("endTime"));
        wrapper.orderBy(true, true, SysRole::getRoleSort);
        return this.list(wrapper);
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     *
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRolesByUserId (Long userId) {
        List<SysRole> userRoles = roleMapper.selectRolePermissionByUserId(userId);
        List<SysRole> roles = selectRoleAll();
        for (SysRole role : roles) {
            for (SysRole userRole : userRoles) {
                if (role.getRoleId().longValue() == userRole.getRoleId().longValue()) {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     *
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId (Long userId) {
        List<SysRole> perms = roleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (StringUtils.isNotNull(perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 查询所有角色
     *
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRoleAll () {
        return SpringUtils.getAopProxy(this).selectRoleList(new SysRole());
    }

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     *
     * @return 选中角色ID列表
     */
    @Override
    public List<Long> selectRoleListByUserId (Long userId) {
        return roleMapper.selectRoleListByUserId(userId);
    }

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     *
     * @return 角色对象信息
     */
    @Override
    public SysRole selectRoleById (Long roleId) {
        return roleMapper.selectRoleById(roleId);
    }

    /**
     * 校验角色名称是否唯一
     *
     * @param role 角色信息
     *
     * @return 结果
     */
    @Override
    public boolean checkRoleNameUnique (SysRole role) {
        Long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysRole info = roleMapper.checkRoleNameUnique(role.getRoleName());
        if (StringUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色权限是否唯一
     *
     * @param role 角色信息
     *
     * @return 结果
     */
    @Override
    public boolean checkRoleKeyUnique (SysRole role) {
        Long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysRole info = roleMapper.checkRoleKeyUnique(role.getRoleKey());
        if (StringUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色是否允许操作
     *
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed (SysRole role) {
        if (StringUtils.isNotNull(role.getRoleId()) && role.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
    }

    /**
     * 校验角色是否有数据权限
     *
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope (Long roleId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            SysRole role = new SysRole();
            role.setRoleId(roleId);
            List<SysRole> roles = SpringUtils.getAopProxy(this).selectRoleList(role);
            if (StringUtils.isEmpty(roles)) {
                throw new ServiceException("没有权限访问角色数据！");
            }
        }
    }

    /**
     * 通过角色ID查询角色使用数量
     *
     * @param roleId 角色ID
     *
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId (Long roleId) {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    /**
     * 新增保存角色信息
     *
     * @param role 角色信息
     *
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertRole (SysRole role) {
        // 新增角色信息
        roleMapper.insertRole(role);
        return insertRoleMenu(role);
    }
    /**
     * 修改保存角色信息
     *
     * @param sysRole 角色信息
     *
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateRole (SysRole sysRole) {
        LambdaUpdateWrapper<SysRole> updateWrapper = new LambdaUpdateWrapper<>();

        // 条件判断并更新字段
        updateWrapper.eq(SysRole::getRoleId, sysRole.getRoleId());

        if (sysRole.getRoleName() != null && !sysRole.getRoleName().isEmpty()) {
            updateWrapper.set(SysRole::getRoleName, sysRole.getRoleName());
        }
        if (sysRole.getRoleKey() != null && !sysRole.getRoleKey().isEmpty()) {
            updateWrapper.set(SysRole::getRoleKey, sysRole.getRoleKey());
        }
        if (sysRole.getRoleSort() != null) {
            updateWrapper.set(SysRole::getRoleSort, sysRole.getRoleSort());
        }
        if (sysRole.getDataScope() != null && !sysRole.getDataScope().isEmpty()) {
            updateWrapper.set(SysRole::getDataScope, sysRole.getDataScope());
        }
        if (sysRole.getStatus() != null && !sysRole.getStatus().isEmpty()) {
            updateWrapper.set(SysRole::getStatus, sysRole.getStatus());
        }
        if (sysRole.getRemark() != null) {
            updateWrapper.set(SysRole::getRemark, sysRole.getRemark());
        }
        if (sysRole.getUpdateBy() != null && !sysRole.getUpdateBy().isEmpty()) {
            updateWrapper.set(SysRole::getUpdateBy, sysRole.getUpdateBy());
        }

        // 更新时间
        updateWrapper.set(SysRole::getUpdateTime, new java.util.Date());

        // 执行更新
         this.update(updateWrapper);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(sysRole.getRoleId());
        return insertRoleMenu(sysRole);
    }

    /**
     * 修改角色状态
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean updateRoleStatus (SysRole role) {
        // 通过 roleId 更新角色信息
        return updateById(role);
    }

    /**
     * 修改数据权限信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean authDataScope (SysRole role) {
        // 修改角色信息
        // 通过 roleId 更新角色信息
        boolean updateResult = updateById(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        // 新增角色和部门信息（数据权限）
        return updateResult;
    }

    /**
     * 新增角色菜单信息
     *
     * @param role 角色对象
     */
    public int insertRoleMenu (SysRole role) {
        // 假设role.getMenuIds()返回角色关联的菜单ID列表
        List<Long> menuIds = List.of(role.getMenuIds());
        if (menuIds == null || menuIds.isEmpty()){
            return 0;
        }
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<SysRoleMenu>();
        for (Long menuId : role.getMenuIds()) {
            SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(role.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        // 使用MyBatis-Plus的saveBatch方法批量插入角色与菜单的关联关系
        boolean saveResult = roleMenuMapper.saveBatch(list);

        // 返回成功插入的数量
        return saveResult ? list.size() : 0;
    }

    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    public int insertRoleDept (SysRole role) {
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysRoleDept> list = new ArrayList<SysRoleDept>();
        for (Long deptId : role.getDeptIds()) {
            SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0) {
            rows = roleDeptMapper.batchRoleDept(list);
        }
        return rows;
    }

    /**
     * 通过角色ID删除角色
     *
     * @param roleId 角色ID
     *
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleById (Long roleId) {
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(roleId);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(roleId);
        return roleMapper.deleteRoleById(roleId);
    }

    /**
     * 批量删除角色信息
     *
     * @param roleIds 需要删除的角色ID
     *
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleByIds (Long[] roleIds) {
        for (Long roleId : roleIds) {
            checkRoleAllowed(new SysRole(roleId));
            checkRoleDataScope(roleId);
            SysRole role = selectRoleById(roleId);
            if (countUserRoleByRoleId(roleId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDept(roleIds);
        return roleMapper.deleteRoleByIds(roleIds);
    }

    /**
     * 取消授权用户角色
     *
     * @param userRole 用户和角色关联信息
     *
     * @return 结果
     */
    @Override
    public int deleteAuthUser (SysUserRole userRole) {
        return userRoleMapper.deleteUserRoleInfo(userRole);
    }

    /**
     * 批量取消授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要取消授权的用户数据ID
     *
     * @return 结果
     */
    @Override
    public int deleteAuthUsers (Long roleId, Long[] userIds) {
        return userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }

    /**
     * 批量选择授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要授权的用户数据ID
     *
     * @return 结果
     */
    @Override
    public int insertAuthUsers (Long roleId, Long[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<SysUserRole>();
        for (Long userId : userIds) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return userRoleMapper.batchUserRole(list);
    }

    @Override
    public PageQueryModel<SysRole> pageQuery(SysRolePageQueryModel sysRolePageQueryModel) {
        LambdaQueryWrapper<SysRole> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(sysRolePageQueryModel.getRoleName()),SysRole::getRoleName, sysRolePageQueryModel.getRoleName());
        queryWrapper.like(StringUtils.isNotEmpty(sysRolePageQueryModel.getRoleKey()),SysRole::getRoleKey, sysRolePageQueryModel.getRoleKey());
        queryWrapper.eq(Objects.nonNull(sysRolePageQueryModel.getStatus()),SysRole::getStatus, sysRolePageQueryModel.getStatus());
        queryWrapper.gt(Objects.nonNull(sysRolePageQueryModel.getBeginTime()),SysRole::getCreateTime, sysRolePageQueryModel.getBeginTime());
        queryWrapper.lt(Objects.nonNull(sysRolePageQueryModel.getEndTime()),SysRole::getCreateTime, sysRolePageQueryModel.getEndTime());
        Page<SysRole> page = this.page(sysRolePageQueryModel.buildPage(), queryWrapper);
        return PageQueryModel.of(page);
    }


}
